@@ -39,7 +39,7 @@
 //! The next output of a value-menu corresponds to its next fields, so if it is, for example, a
 //! selectable menu field, it will display the list of output values, then return the value the user
 //! selected. Attention: if all the fields have been retrieved, the value-menu will be empty, and the
-//! next call of this method will return an error (See [`MenuError::NoMoreField`](crate::MenuError::NoMoreField)).
+//! next call of this method will panic.
 //!
 //! Therefore, a selectable menu can return many times the value selected by the user at different
 //! points of the code.
@@ -85,12 +85,40 @@
 //! let ty: Type = license.next_output().unwrap();
 //! ```
 
+mod stream;
+
+pub use crate::menu::stream::MenuStream;
+use crate::menu::stream::Stream;
 use crate::prelude::*;
-use std::array::IntoIter;
 use std::fmt::{self, Debug, Display, Formatter};
-use std::io::{stdin, stdout, Stdin, Stdout, Write};
+use std::io::{BufRead, BufReader, Stdin, Stdout, Write};
 use std::rc::Rc;
 use std::str::FromStr;
+use std::vec::IntoIter;
+
+/// The default input stream used by a menu, using the standard input stream.
+pub type In = BufReader<Stdin>;
+
+/// The default output stream used by a menu, using the standard output stream.
+pub type Out = Stdout;
+
+/// Used to retrieve the stream contained in a menu.
+pub trait GetStream<'s, R: 's, W: 's>: Sized {
+    /// Returns the menu stream, consuming the menu.
+    fn get_stream(self) -> MenuStream<'s, R, W>;
+
+    /// Returns the input and output streams, consuming the menu.
+    /// See [`MenuStream::retrieve`](stream::MenuStream::retrieve) for more information.
+    ///
+    /// ## Panics
+    ///
+    /// If it hasn't been given a reader and a writer, this method will panic, because it needs
+    /// to own the reader and writer to retrieve it at the end.
+    #[inline]
+    fn retrieve(self) -> (R, W) {
+        self.get_stream().retrieve()
+    }
+}
 
 /// The position of the title for an enum menu.
 /// By default, the title position is at the top.
@@ -164,10 +192,10 @@ impl Default for TitlePos {
 ///     }
 /// }
 ///
-/// let license_type = SelectMenu::from([
-///     SelectField::new("MIT"),
-///     SelectField::new("GPL"),
-///     SelectField::new("BSD"),
+/// let license_type: Type = SelectMenu::from([
+///     SelectField::from("MIT"),
+///     SelectField::from("GPL"),
+///     SelectField::from("BSD"),
 /// ])
 /// .title(SelectTitle::from("Choose a license type"))
 /// .default(0)
@@ -187,11 +215,10 @@ impl Default for TitlePos {
 /// ```
 ///
 /// The default chip is `" - "`, and the default prefix is `">> "`.
-pub struct SelectMenu<'a> {
+pub struct SelectMenu<'a, R = In, W = Out> {
     title: SelectTitle<'a>,
-    fields: Vec<SelectField<'a>>,
-    reader: Stdin,
-    writer: Stdout,
+    fields: Vec<SelectField<'a, R, W>>,
+    stream: Stream<'a, MenuStream<'a, R, W>>,
     default: Option<usize>,
     prefix: &'a str,
 }
@@ -307,25 +334,60 @@ impl Display for SelectTitle<'_> {
 }
 
 impl<'a> From<Vec<SelectField<'a>>> for SelectMenu<'a> {
+    /// Builds the menu from its fields vector.
+    #[inline]
     fn from(fields: Vec<SelectField<'a>>) -> Self {
-        Self {
-            title: Default::default(),
-            fields,
-            reader: stdin(),
-            writer: stdout(),
-            default: None,
-            prefix: ">> ",
-        }
+        Self::with_owned(MenuStream::default(), fields)
     }
 }
 
 impl<'a, const N: usize> From<[SelectField<'a>; N]> for SelectMenu<'a> {
+    /// Builds the menu from an array of fields.
+    #[inline]
     fn from(fields: [SelectField<'a>; N]) -> Self {
         Self::from(Vec::from(fields))
     }
 }
 
-impl<'a> SelectMenu<'a> {
+impl<'a, R, W> SelectMenu<'a, R, W> {
+    fn inner_new(
+        stream: Stream<'a, MenuStream<'a, R, W>>,
+        fields: Vec<SelectField<'a, R, W>>,
+    ) -> Self {
+        Self {
+            title: Default::default(),
+            fields,
+            stream,
+            default: None,
+            prefix: ">> ",
+        }
+    }
+
+    /// Builds the menu from its owned menu stream, with its fields vector.
+    #[inline]
+    pub fn with_owned(stream: MenuStream<'a, R, W>, fields: Vec<SelectField<'a, R, W>>) -> Self {
+        Self::inner_new(Stream::Owned(stream), fields)
+    }
+
+    /// Builds the menu from a mutable reference of a menu stream, with its fields vector.
+    #[inline]
+    pub fn with_ref(
+        stream: &'a mut MenuStream<'a, R, W>,
+        fields: Vec<SelectField<'a, R, W>>,
+    ) -> Self {
+        Self::inner_new(Stream::Borrowed(stream), fields)
+    }
+
+    /// Builds the menu from mutable references of the reader and writer, with its fields vector.
+    #[inline]
+    pub fn new_ref(
+        reader: &'a mut R,
+        writer: &'a mut W,
+        fields: Vec<SelectField<'a, R, W>>,
+    ) -> Self {
+        Self::with_owned(MenuStream::with(reader, writer), fields)
+    }
+
     /// Sets the title of the selective menu.
     ///
     /// The title is by default displayed at the top of the selective fields,
@@ -337,11 +399,6 @@ impl<'a> SelectMenu<'a> {
         self
     }
 
-    #[inline]
-    pub(crate) fn inherit_fmt(&mut self, fmt: Rc<ValueFieldFormatting<'a>>) {
-        self.title.inherit_fmt(fmt);
-    }
-
     /// Sets the default selective field.
     ///
     /// If you have specified a default field, the latter will be marked as `"(default)"`.
@@ -370,9 +427,21 @@ impl<'a> SelectMenu<'a> {
         }
         self
     }
+
+    #[inline]
+    pub(crate) fn inherit_fmt(&mut self, fmt: Rc<ValueFieldFormatting<'a>>) {
+        self.title.inherit_fmt(fmt);
+    }
 }
 
-impl Display for SelectMenu<'_> {
+impl<'a, R, W> GetStream<'a, R, W> for SelectMenu<'a, R, W> {
+    #[inline]
+    fn get_stream(self) -> MenuStream<'a, R, W> {
+        self.stream.retrieve()
+    }
+}
+
+impl<R, W> Display for SelectMenu<'_, R, W> {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         // displays title at top
         if let TitlePos::Top = self.title.pos {
@@ -383,11 +452,11 @@ impl Display for SelectMenu<'_> {
         for (i, field) in self.fields.iter().enumerate() {
             writeln!(
                 f,
-                "{i}{msg} {def}",
+                "{i}{msg}{def}",
                 i = i + 1,
                 msg = field,
                 def = if matches!(self.default, Some(d) if d == i) {
-                    "(default)"
+                    " (default)"
                 } else {
                     ""
                 },
@@ -402,22 +471,48 @@ impl Display for SelectMenu<'_> {
     }
 }
 
-impl AsRef<Stdout> for SelectMenu<'_> {
-    fn as_ref(&self) -> &Stdout {
-        &self.writer
-    }
+/// Returns an error meaning that the value type contained in the string slice is incorrect.
+#[inline]
+fn err_ty<E: 'static + Debug>(e: E) -> MenuError {
+    MenuError::from(format!("incorrect default value type: {:?}", e))
 }
 
-impl AsMut<Stdout> for SelectMenu<'_> {
-    fn as_mut(&mut self) -> &mut Stdout {
-        &mut self.writer
-    }
-}
-
-impl<Output> MenuBuilder<Output> for SelectMenu<'_>
+/// Returns the default value among the fields.
+///
+/// If the default value index or the aimed default field type is incorrect,
+/// it will return an error (See [`MenuError::IncorrectType`]).
+fn default_parse<Output, R, W>(
+    default: usize,
+    fields: &[SelectField<'_, R, W>],
+    stream: &mut MenuStream<R, W>,
+) -> MenuResult<Output>
 where
     Output: FromStr,
     Output::Err: 'static + Debug,
+{
+    let field = fields.get(default).unwrap_or_else(|| {
+        default_parse_failed(
+            default,
+            format!(
+                "incorrect index: fields vector length is {}, index is {}",
+                fields.len(),
+                default
+            ),
+        )
+    });
+    field.call_bind(stream)?;
+    Ok(field
+        .msg
+        .parse()
+        .unwrap_or_else(|e| default_parse_failed(default, e)))
+}
+
+impl<Output, R, W> MenuBuilder<Output, R, W> for SelectMenu<'_, R, W>
+where
+    Output: FromStr,
+    Output::Err: 'static + Debug,
+    R: BufRead,
+    W: Write,
 {
     /// Displays the selective menu to the user, then return the field he selected.
     ///
@@ -455,115 +550,183 @@ where
     /// .next_output()
     /// .unwrap();
     /// ```
+    ///
+    /// ## Panic
+    ///
+    /// This method panics if an incorrect index has been specified as default.
     fn next_output(&mut self) -> MenuResult<Output> {
-        /// Returns an error meaning that the default selection index is incorrect.
-        #[inline]
-        fn err_idx(default: usize, len: usize) -> MenuError {
-            MenuError::from(format!(
-                "incorrect default value index: index is {} but selective menu length is {}",
-                default, len
-            ))
-        }
-
-        /// Returns an error meaning that the value type contained in the string slice is incorrect.
-        #[inline]
-        fn err_ty<E: 'static + Debug>(e: E) -> MenuError {
-            MenuError::from(format!("incorrect default value type: {:?}", e))
-        }
-
-        /// Returns the default value among the fields.
-        ///
-        /// If the default value index or the aimed default field type is incorrect,
-        /// it will return an error (See [`MenuError::IncorrectType`]).
-        fn default_parse<Output>(
-            default: usize,
-            fields: &[SelectField<'_>],
-            writer: &mut Stdout,
-        ) -> MenuResult<Output>
-        where
-            Output: FromStr,
-            Output::Err: 'static + Debug,
         {
-            let field = fields
-                .get(default)
-                .ok_or_else(|| err_idx(default, fields.len()))?;
-            field.call_bind(writer)?;
-            field.msg.parse().map_err(err_ty)
+            let s = format!("{}", self);
+            self.stream.write_all(s.as_bytes())?;
         }
 
-        // displays the menu once
-        self.writer.write_all(format!("{}", self).as_bytes())?;
-
         // loops while incorrect input
         loop {
-            // printing prefix
-            self.writer.write_all(self.prefix.as_bytes())?;
-            self.writer.flush()?;
-
-            // reading input
-            let mut out = String::new();
-            self.reader.read_line(&mut out)?;
-            let out = out.trim();
-
-            if let Some(field) = self
-                .fields
-                .iter()
-                .find(|field| field.msg.to_lowercase() == out.to_lowercase())
-            {
-                // value entered as literal
-                match out.parse::<Output>() {
-                    Ok(out) => {
-                        break Ok({
-                            field.call_bind(&mut self.writer)?;
-                            out
-                        })
+            match select(&mut self.stream, self.prefix, self.default, &self.fields) {
+                Ok(out) => break Ok(out),
+                Err(_) => {
+                    if let Some(default) = self.default {
+                        break Ok(default_parse(default, &self.fields, &mut self.stream)?);
                     }
-                    Err(_) => {
-                        if let Some(default) = self.default {
-                            break default_parse(default, &self.fields, &mut self.writer);
-                        }
-                    }
-                }
-            } else {
-                // value entered as index
-                match out.parse::<usize>() {
-                    Ok(idx) if idx >= 1 => {
-                        if let Some(field) = self.fields.get(idx - 1) {
-                            break {
-                                field.call_bind(&mut self.writer)?;
-                                field.msg.parse().map_err(err_ty)
-                            };
-                        }
-                    }
-                    Err(_) => {
-                        if let Some(default) = self.default {
-                            break default_parse(default, &self.fields, &mut self.writer);
-                        }
-                    }
-                    _ => continue,
                 }
             }
         }
     }
+
+    /// Displays the selective menu to the user, then return the field he selected,
+    /// using the given menu stream.
+    ///
+    /// ## Panic
+    ///
+    /// This method panics if an incorrect index has been specified as default.
+    fn next_output_with(&mut self, stream: &mut MenuStream<R, W>) -> MenuResult<Output> {
+        stream.write_all(format!("{}", self).as_bytes())?;
+
+        loop {
+            match select(stream, self.prefix, self.default, &self.fields) {
+                Ok(out) => break Ok(out),
+                Err(_) => {
+                    if let Some(default) = self.default {
+                        break Ok(default_parse(default, &self.fields, stream)?);
+                    }
+                }
+            }
+        }
+    }
+
+    /// Displays the selective menu to the user, then return the field he selected,
+    /// or return the default value of the type specified.
+    ///
+    /// ## Panic
+    ///
+    /// This method panics if an incorrect index has been specified as default.
+    fn next_or_default(&mut self) -> Output
+    where
+        Output: Default,
+    {
+        if {
+            let s = format!("{}", self);
+            self.stream.write_all(s.as_bytes())
+        }
+        .is_ok()
+        {
+            select(&mut self.stream, self.prefix, self.default, &self.fields).unwrap_or_default()
+        } else {
+            Output::default()
+        }
+    }
+
+    /// Displays the selective menu to the user, then return the field he selected,
+    /// or return the default value of the type specified, using the given menu stream.
+    ///
+    /// ## Panic
+    ///
+    /// This method panics if an incorrect index has been specified as default.
+    fn next_or_default_with(&mut self, stream: &mut MenuStream<R, W>) -> Output
+    where
+        Output: Default,
+    {
+        if stream.write_all(format!("{}", self).as_bytes()).is_ok() {
+            select(stream, self.prefix, self.default, &self.fields).unwrap_or_default()
+        } else {
+            Output::default()
+        }
+    }
+}
+
+// We need to use this function from the owned selectable menu stream,
+// or from a provided one, so we don't mutate the whole selectable menu struct.
+fn select<Output, R, W>(
+    stream: &mut MenuStream<R, W>,
+    prefix: &str,
+    default: Option<usize>,
+    fields: &[SelectField<R, W>],
+) -> MenuResult<Output>
+where
+    W: Write,
+    R: BufRead,
+    Output: FromStr,
+    Output::Err: 'static + Debug,
+{
+    stream.write_all(prefix.as_bytes())?;
+    stream.flush()?;
+
+    let out = raw_read_input(stream)?;
+
+    if let Some(field) = fields
+        .iter()
+        .find(|field| field.msg.to_lowercase() == out.to_lowercase())
+    {
+        match out.parse::<Output>() {
+            Ok(out) => {
+                field.call_bind(stream)?;
+                Ok(out)
+            }
+            Err(_) => {
+                if let Some(default) = default {
+                    default_parse(default, fields, stream)
+                } else {
+                    Err(MenuError::Select(out))
+                }
+            }
+        }
+    } else {
+        match out.parse::<usize>() {
+            Ok(idx) if idx >= 1 => {
+                if let Some(field) = fields.get(idx - 1) {
+                    field.call_bind(stream)?;
+                    field.msg.parse().map_err(err_ty)
+                } else {
+                    Err(MenuError::Select(out))
+                }
+            }
+            Err(_) => {
+                if let Some(default) = default {
+                    default_parse(default, fields, stream)
+                } else {
+                    Err(MenuError::Select(out))
+                }
+            }
+            _ => Err(MenuError::Select(out)),
+        }
+    }
 }
 
 /// Represents a value-menu type, which means a menu that retrieves values from the user inputs.
 ///
-/// The `N` const parameter represents the amount of [`ValueField`](crate::field::ValueField)
+/// The `R` type parameter represents its reader type, and the `W` type parameter means its writer type.
+/// By default, it uses the standard input and output streams to get values from the user.
+/// It wraps the streams into a [`MenuStream`].
+///
 /// It has a global formatting applied to the fields it contains by inheritance.
-pub struct ValueMenu<'a, const N: usize> {
+pub struct ValueMenu<'a, R = In, W = Out> {
     title: &'a str,
     fmt: Rc<ValueFieldFormatting<'a>>,
-    fields: IntoIter<Field<'a>, N>,
-    reader: Stdin,
-    writer: Stdout,
-    // used to know when to print the title
-    first_popped: bool,
+    fields: IntoIter<Field<'a, R, W>>,
+    stream: Stream<'a, MenuStream<'a, R, W>>,
+    popped: bool,
 }
 
-impl<'a, const N: usize> From<[Field<'a>; N]> for ValueMenu<'a, N> {
+impl<'a, const N: usize> From<[Field<'a>; N]> for ValueMenu<'a> {
     /// Instantiate the value-menu from its value-fields array.
-    fn from(mut fields: [Field<'a>; N]) -> Self {
+    #[inline]
+    fn from(fields: [Field<'a>; N]) -> Self {
+        Self::from(Vec::from(fields))
+    }
+}
+
+impl<'a> From<Vec<Field<'a>>> for ValueMenu<'a> {
+    #[inline]
+    fn from(fields: Vec<Field<'a>>) -> Self {
+        Self::with_owned(MenuStream::default(), fields)
+    }
+}
+
+impl<'a, R, W> ValueMenu<'a, R, W> {
+    fn inner_new(
+        stream: Stream<'a, MenuStream<'a, R, W>>,
+        mut fields: Vec<Field<'a, R, W>>,
+    ) -> Self {
         let fmt: Rc<ValueFieldFormatting> = Rc::default();
 
         // inherits fmt on submenus title
@@ -575,14 +738,35 @@ impl<'a, const N: usize> From<[Field<'a>; N]> for ValueMenu<'a, N> {
             fields: fields.into_iter(),
             title: "",
             fmt,
-            reader: stdin(),
-            writer: stdout(),
-            first_popped: false,
+            stream,
+            popped: false,
         }
     }
-}
 
-impl<'a, const N: usize> ValueMenu<'a, N> {
+    /// Builds the menu from its owned menu stream, with its fields vector.
+    #[inline]
+    pub fn with_owned(stream: MenuStream<'a, R, W>, fields: Vec<Field<'a, R, W>>) -> Self {
+        Self::inner_new(Stream::Owned(stream), fields)
+    }
+
+    /// Builds the menu from a mutable reference of a menu stream, with its fields vector.
+    #[inline]
+    pub fn with_ref(stream: &'a mut MenuStream<'a, R, W>, fields: Vec<Field<'a, R, W>>) -> Self {
+        Self::inner_new(Stream::Borrowed(stream), fields)
+    }
+
+    /// Builds the menu from its owned input and output streams, with its fields vector.
+    #[inline]
+    pub fn new(reader: R, writer: W, fields: Vec<Field<'a, R, W>>) -> Self {
+        Self::with_owned(MenuStream::new(reader, writer), fields)
+    }
+
+    /// Builds the menu from mutable references of the reader and writer, with its fields vector.
+    #[inline]
+    pub fn new_ref(reader: &'a mut R, writer: &'a mut W, fields: Vec<Field<'a, R, W>>) -> Self {
+        Self::with_owned(MenuStream::with(reader, writer), fields)
+    }
+
     /// Give the global formatting applied to all the fields the menu contains.
     /// If a field has a custom formatting, it will uses the formatting rules of the field
     /// when printing to the writer.
@@ -600,42 +784,141 @@ impl<'a, const N: usize> ValueMenu<'a, N> {
         self.title = title;
         self
     }
+
+    #[inline]
+    fn next_field(&mut self) -> Field<'a, R, W> {
+        self.fields.next().expect("no more field in the value-menu")
+    }
 }
 
 /// Trait used to return the next output of the menu.
-pub trait MenuBuilder<Output>: AsRef<Stdout> + AsMut<Stdout> {
+pub trait MenuBuilder<Output, R, W> {
     /// Returns the next output from the menu.
     fn next_output(&mut self) -> MenuResult<Output>;
-}
 
-impl<const N: usize> AsRef<Stdout> for ValueMenu<'_, N> {
-    fn as_ref(&self) -> &Stdout {
-        &self.writer
+    /// Returns the next output from the menu using a given menu stream in parameter.
+    fn next_output_with(&mut self, stream: &mut MenuStream<R, W>) -> MenuResult<Output>;
+
+    /// Returns the next output from the menu, or its default value if an error occurred.
+    fn next_or_default(&mut self) -> Output
+    where
+        Output: Default,
+    {
+        self.next_output().unwrap_or_default()
+    }
+
+    /// Returns the next output from the menu using a given menu stream in parameter,
+    /// or its default value if an error occurred.
+    fn next_or_default_with(&mut self, stream: &mut MenuStream<R, W>) -> Output
+    where
+        Output: Default,
+    {
+        self.next_output_with(stream).unwrap_or_default()
     }
 }
 
-impl<const N: usize> AsMut<Stdout> for ValueMenu<'_, N> {
-    fn as_mut(&mut self) -> &mut Stdout {
-        &mut self.writer
-    }
-}
-
-impl<Output, const N: usize> MenuBuilder<Output> for ValueMenu<'_, N>
+impl<Output, R, W> MenuBuilder<Output, R, W> for ValueMenu<'_, R, W>
 where
     Output: FromStr,
     Output::Err: 'static + Debug,
+    R: BufRead,
+    W: Write,
 {
     /// Returns the output of the next field if present.
+    ///
+    /// ## Panic
+    ///
+    /// This function panics if there is no more field in the value-menu,
+    /// or if an incorrect value type has been used as default.
     fn next_output(&mut self) -> MenuResult<Output> {
-        // prints the title
-        if !self.first_popped && !self.title.is_empty() {
-            writeln!(self.writer, "{}", self.title)?;
-            self.first_popped = true;
-        }
+        print_title(&mut self.stream, self.title, &mut self.popped)?;
+        self.next_field().menu_build(&mut self.stream)
+    }
 
-        self.fields
-            .next()
-            .ok_or(MenuError::NoMoreField)?
-            .build(&self.reader, &mut self.writer)
+    /// Returns the output of the next field if present using the given menu stream.
+    ///
+    /// ## Panic
+    ///
+    /// This function panics if there is no more field in the value-menu,
+    /// or if an incorrect value type has been used as default.
+    fn next_output_with(&mut self, stream: &mut MenuStream<R, W>) -> MenuResult<Output> {
+        print_title(&mut self.stream, self.title, &mut self.popped)?;
+        self.next_field().menu_build(stream)
+    }
+
+    /// Returns the output of the next field if present or its default type.
+    ///
+    /// ## Panic
+    ///
+    /// This function panics if there is no more field in the value-menu,
+    /// or if an incorrect value type has been used as default.
+    fn next_or_default(&mut self) -> Output
+    where
+        Output: Default,
+    {
+        let field = self.next_field();
+        next_or_default(&mut self.stream, field, &mut self.popped, self.title)
+    }
+
+    /// Returns the output of the next field if present or its default type,
+    /// using the given menu stream.
+    ///
+    /// ## Panic
+    ///
+    /// This function panics if there is no more field in the value-menu,
+    /// or if an incorrect value type has been used as default.
+    fn next_or_default_with(&mut self, stream: &mut MenuStream<R, W>) -> Output
+    where
+        Output: Default,
+    {
+        let field = self.next_field();
+        next_or_default(stream, field, &mut self.popped, self.title)
+    }
+}
+
+fn next_or_default<Output, R, W>(
+    stream: &mut MenuStream<R, W>,
+    mut field: Field<'_, R, W>,
+    popped: &mut bool,
+    title: &str,
+) -> Output
+where
+    Output: FromStr + Default,
+    Output::Err: 'static + Debug,
+    W: Write,
+    R: BufRead,
+{
+    if print_title(stream, title, popped).is_ok() {
+        field.menu_build_or_default(stream)
+    } else {
+        Output::default()
+    }
+}
+
+fn print_title<R, W>(
+    stream: &mut MenuStream<R, W>,
+    title: &str,
+    popped: &mut bool,
+) -> MenuResult<()>
+where
+    W: Write,
+{
+    if !*popped && !title.is_empty() {
+        writeln!(stream, "{}", title)?;
+        *popped = true;
+    }
+    Ok(())
+}
+
+impl<'a, R, W> GetStream<'a, R, W> for ValueMenu<'a, R, W> {
+    /// Returns the menu stream, consuming the menu.
+    ///
+    /// ## Panic
+    ///
+    /// If it hasn't been given a menu stream, this method will panic,
+    /// because it needs to own its stream to retrieve it at the end.
+    #[inline]
+    fn get_stream(self) -> MenuStream<'a, R, W> {
+        self.stream.retrieve()
     }
 }
